@@ -24,10 +24,16 @@ class MyntException(Exception):
         return message
 
 
+class ConfigException(MyntException):
+    pass
+
+class FileSystemException(MyntException):
+    pass
+
 class OptionException(MyntException):
     code = 2
 
-class ConfigException(MyntException):
+class PageException(MyntException):
     pass
 
 class ParserException(MyntException):
