@@ -4,6 +4,7 @@ from __future__ import absolute_import, unicode_literals
 
 import re
 
+import houdini as h
 import misaka as m
 
 from mynt.base import Parser as _Parser
@@ -19,13 +20,12 @@ class _Renderer(m.HtmlRenderer):
         (r'^$', 'section')
     ]
     
+    
     def block_code(self, text, lang):
-        lang = ' lang="{0}"'.format(lang) if lang else ''
+        text = h.escape_html(text.encode('utf-8'), 1).decode('utf-8')
+        lang = ' data-lang="{0}"'.format(lang) if lang else ''
         
-        for pattern, replace in [('&', '&amp;'), ('"', '&#34;'), ('\'', '&#39;'), ('>', '&gt;'), ('<', '&lt;')]:
-                text = text.replace(pattern, replace)
-        
-        return '<pre{0}><code>{1}</code></pre>'.format(lang, text).encode('utf-8')
+        return '<pre><code{0}>{1}</code></pre>'.format(lang, text)
     
     def header(self, text, level):
         if self.flags & m.HTML_TOC:
@@ -40,14 +40,26 @@ class _Renderer(m.HtmlRenderer):
             else:
                 self._toc_ids[identifier] = 1
             
-            return '<h{0} id="{1}">{2}</h{0}>'.format(level, identifier, text).encode('utf-8')
+            return '<h{0} id="{1}">{2}</h{0}>'.format(level, identifier, text)
         else:
-            return '<h{0}>{1}</h{0}>'.format(level, text).encode('utf-8')
+            return '<h{0}>{1}</h{0}>'.format(level, text)
+    
+    
+    def setup(self):
+        super(_Renderer, self).setup()
+        
+        self.sp = m.SmartyPants().postprocess
     
     def preprocess(self, markdown):
         self._toc_ids = {}
         
         return markdown
+    
+    def postprocess(self, html):
+        if self.flags & m.HTML_SMARTYPANTS:
+            html = self.sp(html)
+        
+        return html
 
 
 class Parser(_Parser):
